/**
 * Custom CSS for RepeaterBook documentation
 *
 * Add any custom styles here to override Material theme defaults.
 */

/* Example: Customize code block background
.highlight {
    background-color: #f5f5f5;
}
*/
